// IndexUpdater.cpp
// Uses the generic Volume Recursor to go through a volume or 
// volumes and remove, then recreate, an attribute, in order to
// update the index associated with it. Newly-created indices do
// not contain any prior existing attributes, so it is necessary
// to recreate the attributes so they are added to the index.

#include "IndexUpdater.h"
#include <string.h>
#include <Node.h>
#include <fs_attr.h>
#include <malloc.h>
#include <stdio.h>

void Usage(char* appName);

int
main(int argc, char** argv)
{
	if (argc >= 2)
	{
		printf("Updating index %s on all mounted volumes.\n", argv[1]);
		IndexUpdater* updater = new IndexUpdater(argv[1]);
		updater->Recurse();
		delete updater;
	}
	else
	{
		Usage(argv[0]);
	}
	return 0;
}
void Usage(char* appName)
{
	printf("%s - updates specified index on all mounted volumes\n", appName);
	printf("Usage: %s <index_name>\n", appName);
}

IndexUpdater::IndexUpdater(char* attributeName) 
{
	if (attributeName != NULL)
		mAttributeName = strdup(attributeName);
}

IndexUpdater::~IndexUpdater()
{
	if (mAttributeName != NULL)
		free(mAttributeName);
}
status_t
IndexUpdater::Apply(BEntry* entry)
{
	// Get the named index, read it in, write it
	// back out.
	BNode 		node(entry);
	status_t	result = B_NO_ERROR;
	attr_info	info;
	result = node.GetAttrInfo(mAttributeName, &info);
	if (result == B_NO_ERROR)
	{
		// The attribute exists on this node, so let's remove it
		// and recreate it.
		void *data = malloc(info.size);
		if (node.ReadAttr(mAttributeName, info.type, 0, data, info.size) == info.size)
		{
			printf("Updating attribute\n");
			result = node.RemoveAttr(mAttributeName);
			node.WriteAttr(mAttributeName, info.type, 0, data, info.size);		
		}
	}
	return result;
}
