#include <List.h>
#include <VolumeRoster.h>
#include <string.h>

#include "VolumeRecursor.h"

VolumeRecursor::VolumeRecursor(VolumeRecurseFunc func) :
		mRecurseFunc(func)
{
	return;
}

status_t
VolumeRecursor::Recurse(BVolume* targetVolume)
{
	BList 		volumeList;
	status_t 	result = B_NO_ERROR;
	
	if (targetVolume != NULL)
	{
		result = ApplyToVolume(targetVolume);
	}
	else
	{
		BVolumeRoster  roster;
		BVolume 		volume;
		roster.Rewind();
		while (roster.GetNextVolume(&volume) == B_NO_ERROR)
		{
			char volName[32];
			if (volume.GetName(volName) == B_NO_ERROR)
			{
				if (strcmp(volName, ""))
				{
					BVolume* vol = new BVolume(volume);
					volumeList.AddItem((void*)vol);
				}
			}
		}
	}
	

	for (int32 i = 0; i < volumeList.CountItems();i++)
	{
		BVolume* aVol = (BVolume*)volumeList.ItemAt(i);
		result = ApplyToVolume(aVol);
		if (result != B_NO_ERROR) break;
		delete aVol;
	}
	return result;
}

status_t 
VolumeRecursor::Recurse(BDirectory* directory)
{
	status_t result;
	result = ApplyToDirectory(directory);
	return result;
}
status_t
VolumeRecursor::ApplyToVolume(BVolume* volume)
{
	// Start at root. Recurse
	// through all lower entries. If mRecurseFunc != NULL, call
	// it. Otherwise, call the Apply(BEntry*) method.
	status_t 	result;
	BDirectory 	dir;
	volume->GetRootDirectory(&dir);	
	result = ApplyToDirectory(&dir);
	return result;
}

status_t
VolumeRecursor::ApplyToDirectory(BDirectory* directory)
{
	status_t result = B_NO_ERROR;
	
	directory->Rewind();
	BEntry entry;
	
	while (directory->GetNextEntry(&entry) == B_NO_ERROR)
	{
		if (entry.IsDirectory())
		{
			BDirectory subDir(&entry);
			ApplyToDirectory(&subDir);
		}
		else
		{
			if (mRecurseFunc != NULL)
			{
				mRecurseFunc(&entry);
			}
			else Apply(&entry);
		}
	}
	return result;
}

status_t
VolumeRecursor::Apply(BEntry* entry)
{
	#pragma unused(entry)
	return B_NO_ERROR;
}