// VolumeRecursor.h
/*
	A class that allows you to apply a function to entries on one
	or more volumes. The function called can be specified either
	by setting a VolumeRecurseFunc with the SetRecurseFunc() method,
	or by subclassing VolumeRecursor and implementing the Apply()
	method. 
	The VolumeRecurseFunc takes precedence over the calling of the
	Apply() method, and never will both be called.
	
	Entries affected can be specified in 3 ways when calling the
	Recurse() method to start the recursion:
	
	1. Recurse() causes all entries on all volumes to be affected.
	
	2. Recurse(BVolume*) causes all entries on specified volume
	   to be affected.
	   
	3. Recurse(BVolume*, BDirectory*) causes all entries under
	   the specified directory on specified volume to be affected.
	   
	The initial motivation for this class was to  provide an easy
	way to update indices.
*/
#pragma once
#include <Volume.h>
#include <Entry.h>
#include <Directory.h>

typedef status_t (*VolumeRecurseFunc)(BEntry*);

class VolumeRecursor
{
	public:
							VolumeRecursor(VolumeRecurseFunc func = NULL);
		virtual status_t	Recurse(BVolume* volume = NULL);
		virtual status_t	Recurse(BDirectory* directory);
		
	private:
		virtual status_t	ApplyToVolume(BVolume* volume);
		virtual status_t	ApplyToDirectory(BDirectory* directory);
		virtual status_t	Apply(BEntry* entry);
		VolumeRecurseFunc	mRecurseFunc;
};
